;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       avg_dark_bb
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       dark = make_dark_bb(ddir, dlevel, drms, dtime)
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       dark = averaged dark array
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION avg_dark_bb, ddir, dlevel, drms, dtime

     dfiles = file_search( ddir + '*.DarkCalibration.fits', COUNT = nd)
     sz = read_size(dfiles[0])
     dark = FLTARR(sz[0], sz[1])
     nimages = lonarr(nd)

     FOR k = 0, nd-1 DO BEGIN

         FITS_OPEN, dfiles[k], tmp
         nimages[k] = tmp.nextend
         FITS_CLOSE, tmp

     ENDFOR

     ntotal = total(nimages)

     PRINT,'---------------------------------------'
     PRINT,'Dark files found: ', nd
     PRINT,'Total number of dark images: ', ntotal
     PRINT,'---------------------------------------'

     PRINT & PAUSE,'Enter files not used for average dark (def = use all, enter 1 2 3 4 5 ...)', /nopause, ans = ans

     IF ans EQ '' THEN BEGIN

	PRINT,'Processing all'
       
        n = nd

     ENDIF ELSE BEGIN

           ans = fix(get_words(ans))
           n = nd - n_elements(ans)
           index = ibis_good_frame_index(nd,ans)
           dfiles = dfiles[index]
           nimages = nimages[index]
           ntotal = total(nimages)
   
     ENDELSE
              
     dlevel = FLTARR(ntotal)
     drms   = FLTARR(ntotal)
     dtime  = FLTARR(ntotal)
     
     l = 0

     FOR k = 0, n-1 DO BEGIN

         PRINT,'Dark file : ', dfiles[k]
         FITS_OPEN, dfiles[k], tmp

         FOR i = 1, nimages[k] DO BEGIN
 
             FITS_READ, tmp, data, header, exten_no= i, /NO_PDU
             statist,FLOAT(data)
             dlevel[l] = AVG(FLOAT(data))
             drms[l] = stdev(data)
             dtime[l] = time_conv_new(STRMID(SXPAR(header, 'DATE-OBS'), 11))
             dark += FLOAT(data)
             l = l+1

         ENDFOR
    
         FITS_CLOSE, tmp

         window,0
         plot, dlevel, psym=-4, xtitle = 'image number', ytitle = 'mean dark counts'

     ENDFOR     
         
     window,0
     plot, dtime-dtime[0], dlevel, psym=-4, xtitle = 'time   [sec]', ytitle = 'mean dark counts'
     window,1
     plot, dtime-dtime[0], drms, psym=-4, xtitle = 'time   [sec]', ytitle = 'rms'

     dark /= FLOAT(ntotal)

     PRINT,'----------------------------'
     PRINT,'statistics mean dark :      '
     PRINT,'----------------------------'

     statist,dark

     RETURN, dark
END
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       avg_flat_bb
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       flat = avg_flat_bb(fdir,dark, fllevel, flevel, frms, ftime)
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       flat =     averaged flat array
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION avg_flat_bb, fdir, dark, fllevel, flevel, frms, ftime

     ffiles = file_search( fdir + '*.FlatFieldCalibration.fits', COUNT = nd)
     sz = read_size(ffiles[0])
     flat = FLTARR(sz[0], sz[1])
     nimages = lonarr(nd)

     FOR k = 0, nd-1 DO BEGIN

         FITS_OPEN, ffiles[k], tmp
         nimages[k] = tmp.nextend
         FITS_CLOSE, tmp

     ENDFOR

     ntotal = total(nimages)

     PRINT,'---------------------------------------'
     PRINT,'Flat files found: ', nd
     PRINT,'Total number of flat images: ', ntotal
     PRINT,'---------------------------------------'

     PRINT & PAUSE,'Enter files not used for average flat (def = use all, enter 0 1 2 3 4 5 ...)', /nopause, ans = ans

     IF ans EQ '' THEN BEGIN

	PRINT,'Processing all'
       
        n = nd

     ENDIF ELSE BEGIN

           ans = fix(get_words(ans))
           n = nd - n_elements(ans)
           index = ibis_good_frame_index(nd,ans)
           ffiles = ffiles[index]
           nimages = nimages[index]
           ntotal = total(nimages)

     ENDELSE
         
     flevel  = FLTARR(ntotal)
     frms    = FLTARR(ntotal)
     ftime   = FLTARR(ntotal)
     fllevel = FLTARR(ntotal)

     l = 0

     FOR k = 0, n-1 DO BEGIN

         print,'Flat file : ', ffiles[k]
         FITS_OPEN, ffiles[k], tmp

         FOR i = 1, nimages[k] DO BEGIN

             FITS_READ, tmp, data, header, exten_no = i, /NO_PDU
             flevel[l] = AVG(FLOAT(data))
             fllevel[l] = SXPAR(header, 'DST_LLVL')
             frms[l] = stdev(data)/avg(data)
             ftime[l] = time_conv_new(STRMID(SXPAR(header, 'DATE-OBS'), 11))
             flat += FLOAT(data-dark)
             l = l+1

         ENDFOR

         FITS_CLOSE, tmp

         window,0
         plot, flevel, psym=-4, xtitle = 'image number', ytitle = 'mean dark counts'

     ENDFOR     
         
     window,0
     plot, ftime-ftime[0], flevel, psym=-4, xtitle = 'time   [sec]', ytitle = 'mean flat counts'
     window,1
     plot, ftime-ftime[0], frms, psym=-4, xtitle = 'time   [sec]', ytitle = 'rms'
     window,2
     plot, ftime-ftime[0], fllevel, psym=-4, xtitle = 'time   [sec]', ytitle = 'DST guider light level'

     flat /= FLOAT(ntotal)

     PRINT,'----------------------------'
     PRINT,'statistics mean flat :      '
     PRINT,'----------------------------'

     statist,flat

     RETURN, flat
END

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_flat_dark_avg_bb
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       ibis_flat_dark_avg_bb, flat_dir, dark_dir, offile, odfile
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_flat_dark_avg_bb, flat_dir, dark_dir, offile, odfile

;;***************************************************************
;; create mean dark if necessary
;;***************************************************************

     IF (FILE_TEST(odfile)) THEN BEGIN
 
        PRINT,'----------------------------------'
        PRINT, 'Mean dark file already exists    '
        PRINT,'----------------------------------'

        RESTORE, odfile, /verb

     ENDIF ELSE BEGIN

           dark = avg_dark_bb(dark_dir, dark_level, dark_rms, dark_time)

           PRINT,'------------------------------'
           PRINT,'save dark: ', odfile
           PRINT,'------------------------------'

           SAVE, dark, dark_level, dark_rms, dark_time, FILENAME = odfile

     ENDELSE

     PRINT,''

;;***************************************************************
;; create mean flat if necessary
;;***************************************************************

     IF (FILE_TEST(offile)) THEN BEGIN

        PRINT,'-----------------------------------'
        PRINT, 'Mean flat file already exists  !! '
        PRINT,'-----------------------------------'

     ENDIF ELSE BEGIN

           flat = avg_flat_bb(flat_dir, dark, flat_llevel, flat_level, flat_rms, flat_time)

           PRINT,'------------------------------'
           PRINT,'save flat: ', offile
           PRINT,'------------------------------'

           SAVE, flat, flat_llevel, flat_level, flat_rms, flat_time, FILENAME = offile
 
     ENDELSE

     PRINT,''

;;***************************************************************
;; Done!
;;***************************************************************

END
